/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: QueryService.java,v 1.6 2003/08/07 20:13:20 vgritsenko Exp $
 */

package org.apache.xindice.client.xmldb.services;

import org.apache.xindice.client.xmldb.CommonConfigurable;
import org.apache.xindice.client.xmldb.XindiceCollection;

import org.xmldb.api.base.Collection;
import org.xmldb.api.base.ResourceSet;
import org.xmldb.api.base.Service;
import org.xmldb.api.base.XMLDBException;

import java.util.Hashtable;

/**
 * Abstract query service.
 *
 * Used to implemented both XUpdate and XPath query
 * service, as the interface for these queries on the XML-RPC API is more
 * or less identical.
 *
 * @author <a href="mailto:james.bates@amplexor.com">James Bates</a>
 * @version CVS $Revision: 1.6 $, $Date: 2003/08/07 20:13:20 $
 */
public abstract class QueryService extends CommonConfigurable implements Service {

    /* XPath or XUpdate. This property should be set by subclasses */
    protected String queryLang;

    /* Collection that this query service should query in */
    Collection collection;

    /* namespace bindings in use for this query */
    Hashtable nsMap = new Hashtable();

    public String getVersion() {

        return "1.0";
    }

    public String getName() {

        return queryLang + "QueryService";
    }

    public void setCollection(Collection collection) {

        this.collection = collection;
    }

    public ResourceSet queryResource(String name, String query)
            throws XMLDBException {

        return ((XindiceCollection) collection).query(name, queryLang, query, nsMap);
    }

    public void clearNamespaces() {

        nsMap.clear();
    }

    public String getNamespace(String prefix) {

        return (String) nsMap.get(prefix);
    }

    public void setNamespace(String prefix, String namespaceURI) {

        nsMap.put(prefix, namespaceURI);
    }

    public ResourceSet query(String query) throws XMLDBException {

        return ((XindiceCollection) collection).query(queryLang, query, nsMap);
    }

    public void removeNamespace(String prefix) throws XMLDBException {

        nsMap.remove(prefix);
    }

}
