/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: XUpdateQueryServiceImpl.java,v 1.10 2003/08/07 20:13:20 vgritsenko Exp $
 */

package org.apache.xindice.client.xmldb.services;

import org.w3c.dom.Document;
import org.xmldb.api.base.ErrorCodes;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.XMLResource;
import org.xmldb.api.modules.XUpdateQueryService;

/**
 * XML:DB XUpdateQueryService implementation that uses XML-RPC communication
 * with server
 *
 * @author <a href="mailto:james.bates@amplexor.com">James Bates</a>
 * @version CVS $Revision: 1.10 $, $Date: 2003/08/07 20:13:20 $
 */
public class XUpdateQueryServiceImpl extends QueryService implements XUpdateQueryService {

    /**
     * Creates new XUpdateQueryService
     */
    public XUpdateQueryServiceImpl() {

        super();
        queryLang = "XUpdate";
    }

    /**
     * Extracts modified count from the pseudo-result returned from query
     */
    private long getResultCount(XMLResource resource) throws XMLDBException {

        Document doc = (Document) resource.getContentAsDOM();

        /**
         modified by tlr - server sends:
         * <src:modified xmlns:src="http://xml.apache.org/xindice/Query">1</src:modified>
         * which is not properly decoded by original code (drills one level too deep)
         // Get the src:modified element
         Node node = doc.getDocumentElement().getFirstChild();

         // The count is a text node within the element.
         String count = node.getFirstChild().getNodeValue();
         **/
        String count = null;
        try {
            count = doc.getDocumentElement().getFirstChild().getNodeValue();
        } catch (Exception e) {
            throw new XMLDBException(ErrorCodes.VENDOR_ERROR, "Unable to retrieve <src:modified> element content from server result", e);
        }
        try {

            return Long.parseLong(count);
        } catch (Exception e) {

            throw new XMLDBException(ErrorCodes.VENDOR_ERROR, "<src:modified> in server result did not contain a valid count", e);
        }
    }

    /**
     * Performs an XUpdate operation on the entire collection
     */
    public long update(String query) throws XMLDBException {

        return getResultCount((XMLResource) query(query).getResource(0));
    }

    /**
     * Preforms an XUpdate operation on the specified document
     */
    public long updateResource(String name, String query) throws XMLDBException {

        return getResultCount((XMLResource) queryResource(name, query).getResource(0));
    }
}
