/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: FaultCodes.java,v 1.18 2003/12/11 11:46:50 vgritsenko Exp $
 */

package org.apache.xindice.core;

import org.xmldb.api.base.ErrorCodes;
import org.xmldb.api.base.XMLDBException;

import java.util.HashMap;
import java.util.Map;

/**
 * FaultCodes defines the Xindice specific fault codes and associated error
 * messages.
 *
 * @version CVS $Revision: 1.18 $, $Date: 2003/12/11 11:46:50 $
 */
public abstract class FaultCodes {

    private static final Map faultMessages = new HashMap();

    public static final int GEN  =    0;
    public static final int OBJ  =  100;
    public static final int COL  =  200;
    public static final int IDX  =  300;
    public static final int TRX  =  400;
    public static final int DBE  =  500;
    public static final int QRY  =  600;
    public static final int SEC  =  700;
    public static final int URI  =  800;
    public static final int JAVA = 2000;

    public static final int GEN_UNKNOWN        =  0;
    public static final int GEN_GENERAL_ERROR  = 40;
    public static final int GEN_CRITICAL_ERROR = 70;
    public static final int GEN_FATAL_ERROR    = 90;

    public static final int COL_COLLECTION_NOT_FOUND = 200;
    public static final int COL_DOCUMENT_NOT_FOUND   = 201;
    public static final int COL_DUPLICATE_COLLECTION = 240;
    public static final int COL_NULL_RESULT          = 241;
    public static final int COL_NO_FILER             = 242;
    public static final int COL_NO_INDEXMANAGER      = 242;
    public static final int COL_DOCUMENT_MALFORMED   = 243;
    public static final int COL_CANNOT_STORE         = 244;
    public static final int COL_CANNOT_RETRIEVE      = 245;
    public static final int COL_COLLECTION_READ_ONLY = 246;
    public static final int COL_COLLECTION_CLOSED    = 247;
    public static final int COL_CANNOT_CREATE        = 270;
    public static final int COL_CANNOT_DROP          = 271;
    public static final int COL_INVALID_RESULT       = 277;

    public static final int IDX_VALUE_NOT_FOUND      = 300;
    public static final int IDX_INDEX_NOT_FOUND      = 301;
    public static final int IDX_MATCHES_NOT_FOUND    = 340;
    public static final int IDX_DUPLICATE_INDEX      = 341;
    public static final int IDX_NOT_SUPPORTED        = 370;
    public static final int IDX_STYLE_NOT_FOUND      = 371;
    public static final int IDX_CORRUPTED            = 372;
    public static final int IDX_CANNOT_CREATE        = 373;

    public static final int TRX_DOC_LOCKED           = 400;
    public static final int TRX_NO_CONTEXT           = 440;
    public static final int TRX_NOT_ACTIVE           = 441;
    public static final int TRX_NOT_SUPPORTED        = 470;

    public static final int DBE_NO_PARENT            = 500;
    public static final int DBE_CANNOT_DROP          = 570;
    public static final int DBE_CANNOT_CREATE        = 571;
    public static final int DBE_CANNOT_READ          = 572;

    public static final int QRY_NULL_RESULT          = 600;
    public static final int QRY_COMPILATION_ERROR    = 640;
    public static final int QRY_PROCESSING_ERROR     = 641;
    public static final int QRY_NOT_SUPPORTED        = 670;
    public static final int QRY_STYLE_NOT_FOUND      = 671;

    public static final int SEC_INVALID_USER         = 770;
    public static final int SEC_INVALID_GROUP        = 771;
    public static final int SEC_INVALID_ACCESS       = 772;
    public static final int SEC_INVALID_CREDENTIALS  = 773;

    public static final int URI_EMPTY                = 800;
    public static final int URI_NULL                 = 801;
    public static final int URI_PARSE_ERROR          = 820;

    public static final int JAVA_RUNTIME_ERROR       = 2070;


    private FaultCodes() {
    }

    static {
        // General errors 0 series
        putCodeMessage(GEN_UNKNOWN, "Unknown");
        putCodeMessage(GEN_GENERAL_ERROR, "General Error");
        putCodeMessage(GEN_CRITICAL_ERROR, "Critical Error");
        putCodeMessage(GEN_FATAL_ERROR, "Fatal Error");

        // Collection-related errors 200 series
        putCodeMessage(COL_COLLECTION_NOT_FOUND, "Collection Not Found");
        putCodeMessage(COL_DOCUMENT_NOT_FOUND, "Collection Document Not Found");
        putCodeMessage(COL_DUPLICATE_COLLECTION, "Collection Duplicated");
        putCodeMessage(COL_NULL_RESULT, "Collection Null Result");
        putCodeMessage(COL_NO_FILER, "Collection No Filer");
        putCodeMessage(COL_NO_INDEXMANAGER, "Collection No IndexManager");
        putCodeMessage(COL_DOCUMENT_MALFORMED, "Collection Document Malformed");
        putCodeMessage(COL_CANNOT_STORE, "Collection Cannot Store");
        putCodeMessage(COL_CANNOT_RETRIEVE, "Collection Cannot Retrieve");
        putCodeMessage(COL_COLLECTION_READ_ONLY, "Collection Read-only");
        putCodeMessage(COL_COLLECTION_CLOSED, "Collection Closed");
        putCodeMessage(COL_CANNOT_CREATE, "Collection Cannot Create");
        putCodeMessage(COL_CANNOT_DROP, "Collection Cannot Drop");
        putCodeMessage(COL_INVALID_RESULT, "Collection Invalid Result");

        // Index-related errors 300 series
        putCodeMessage(IDX_VALUE_NOT_FOUND, "Index Value Not Found");
        putCodeMessage(IDX_INDEX_NOT_FOUND, "Index Not Found");
        putCodeMessage(IDX_MATCHES_NOT_FOUND, "Index Matches Not Found");
        putCodeMessage(IDX_DUPLICATE_INDEX, "Index Duplicate Index");
        putCodeMessage(IDX_NOT_SUPPORTED, "Index Not Supported");
        putCodeMessage(IDX_STYLE_NOT_FOUND, "Index Style Not Found");
        putCodeMessage(IDX_CORRUPTED, "Index Corrupted");
        putCodeMessage(IDX_CANNOT_CREATE, "Index Cannot Create");

        // Transaction-related errors 400 series
        putCodeMessage(TRX_DOC_LOCKED, "Transaction Document Locked");
        putCodeMessage(TRX_NO_CONTEXT, "Transaction No Context");
        putCodeMessage(TRX_NOT_ACTIVE, "Transaction Not Active");
        putCodeMessage(TRX_NOT_SUPPORTED, "Transaction Not Supported");

        // Database-related errors 500 series
        putCodeMessage(DBE_NO_PARENT, "Database No Parent");
        putCodeMessage(DBE_CANNOT_DROP, "Database Cannot Drop");
        putCodeMessage(DBE_CANNOT_CREATE, "Database Cannot Create");
        putCodeMessage(DBE_CANNOT_READ, "Database Cannot Read");

        // Query-related errors 600 series
        putCodeMessage(QRY_NULL_RESULT, "Query Null Result");
        putCodeMessage(QRY_COMPILATION_ERROR, "Query Compilation Error");
        putCodeMessage(QRY_PROCESSING_ERROR, "Query Processing Error");
        putCodeMessage(QRY_NOT_SUPPORTED, "Query Not Supported");
        putCodeMessage(QRY_STYLE_NOT_FOUND, "Query Style Not Found");

        // Security-related errors 700 series
        putCodeMessage(SEC_INVALID_USER, "Security Invalid User");
        putCodeMessage(SEC_INVALID_GROUP, "Security Invalid Group");
        putCodeMessage(SEC_INVALID_ACCESS, "Security Invalid Access");
        putCodeMessage(SEC_INVALID_CREDENTIALS, "Security Invalid Credentials");

        // URI-related errors 800 series
        putCodeMessage(URI_EMPTY, "URI Empty");
        putCodeMessage(URI_NULL, "URI Null");
        putCodeMessage(URI_PARSE_ERROR, "URI Parse Error");

        // Java-related errors 2000 series
        putCodeMessage(JAVA_RUNTIME_ERROR, "Java Runtime Error");
    }

    private static void putCodeMessage(int code, String message) {
        faultMessages.put(new Integer(code), message);
    }

    /**
     * getMessage returns a textual form for the specified fault code.
     *
     * @param code The Fault Code
     * @return It's textual form
     */
    public static String getMessage(int code) {
        String msg = (String) faultMessages.get(new Integer(code));
        return msg != null ? msg : "";
    }

    /**
     * createXMLDBException creates an XMLDBException instance based
     * on the specified Exception.  If the Exception is a DBException,
     * it will extract any important information from it (like fault
     * codes and messages)
     *
     * @param e The Exception to use
     * @return An XMLDBException instance
     */
    public static XMLDBException createXMLDBException(Exception e) {
        return createXMLDBException(null, e);
    }

    /**
     * createXMLDBException creates an XMLDBException instance based
     * on the specified Exception.  If the Exception is a DBException,
     * it will extract any important information from it (like fault
     * codes and messages)
     *
     * @param e The Exception to use
     * @param message The Message to use
     * @return An XMLDBException instance
     */
    public static XMLDBException createXMLDBException(String message, Exception e) {

        int faultCode = e instanceof RuntimeException ? JAVA_RUNTIME_ERROR : GEN_UNKNOWN;
        return createXMLDBException(faultCode, message, e);
    }

    public static XMLDBException createXMLDBException(int faultCode, String message, Exception e) {
        return createXMLDBException(ErrorCodes.VENDOR_ERROR, faultCode, message, e);
    }

    /**
     * Creates an XMLDBException instance based
     * on the specified fault code, message, and Exception.  If the Exception is a DBException,
     * it will extract any important information from it (like fault
     * codes and messages). If the exception is an instance of XindiceException,
     * the exception class name and message of any Throwable(s) available
     * via XindiceException.getCause will also be incorporated
     * into the message. If a chain of wrapped XindiceExceptions is available,
     * the chain will be followed incorporating the class name and message
     * at each level.
     *
     * @param faultCode the fault code for the created exception
     * @param e The Exception to use
     * @return An XMLDBException instance
     */
    public static XMLDBException createXMLDBException(int errorCode, int faultCode, String message, Exception e) {
        if (e instanceof XMLDBException) {
            return (XMLDBException) e;
        }

        if (e instanceof DBException) {
            faultCode = ((DBException) e).faultCode;
        }

        if (message == null) {
            message = e.getMessage();
        } else {
            message += ": " + e.getMessage();
        }

        // TODO: Mapping FaultCodes.COL_DOCUMENT_NOT_FOUND -> ErrorCodes.NO_SUCH_RESOURCE etc

        return new XMLDBException(errorCode, faultCode, message, e);
    }

    /**
     * getFaultCodeType examines the provided exception to determine
     * the general fault code that is associated with it.  General
     * fault codes are reduced from actual fault codes to be one of
     * the GEN_ prefixed fault code values.
     *
     * @param e The Exception to examine
     * @return The General Fault Code
     */
    public static int getFaultCodeType(Exception e) {
        int code = 0;
        if (e instanceof DBException) {
            code = ((DBException) e).faultCode;
        }

        // Strip it to the General series
        code = code % 100;
        // Narrow to a General value
        code = code - (code % 10);
        return code;
    }

    /**
     * getFaultCodeSeries examines the provided exception to
     * determine the fault code series that is associated with it.
     * Series are reduced from actual fault codes to be one of
     * the fault code prefixes (ex: COL, DB, SEC).
     *
     * @param e The Exception to examine
     * @return The Fault Code Series
     */
    public static int getFaultCodeSeries(Exception e) {
        int code = 0;
        if (e instanceof DBException) {
            code = ((DBException) e).faultCode;
        }

        // Strip it to the series
        code = code - (code % 100);
        return code;
    }

    /**
     * getFaultCode examines the provided exception to determine
     * the fault code that is associated with it.
     *
     * @param e The Exception to examine
     * @return The Fault Code
     */
    public static int getFaultCode(Exception e) {
        if (e instanceof DBException) {
            return ((DBException) e).faultCode;
        } else {
            return 0;
        }
    }

    /**
     * getFaultMessage examines the provided exception to determine
     * the fault message that is associated with it.
     *
     * @param e The Exception to examine
     * @return The Fault Message
     */
    public static String getFaultMessage(Exception e) {
        return getMessage(getFaultCode(e));
    }
}
