/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: NullReader.java,v 1.4 2003/12/25 20:04:40 vgritsenko Exp $
 */

package org.apache.xindice.core.meta.inline;

import org.apache.xindice.core.FaultCodes;

/**
 * Read metadata of length zero.  Handy for comparing performance
 * of the metadata-free database with a database using the metadata
 * machinery but containing no metadata.
 *
 * @version CVS $Revision: 1.4 $, $Date: 2003/12/25 20:04:40 $
 */
public class NullReader implements InlineMetaReader {

    /**
     * @see org.apache.xindice.core.meta.inline.InlineMetaReader#getVersion()
     */
    public int getVersion() {
        return 0;
    }

    /**
     * @see org.apache.xindice.core.meta.inline.InlineMetaReader#read(byte[], int, int)
     */
    public InlineMetaMap read(byte[] data, int offset, int length) throws InlineMetaException {

        if (length != 0) {
            throw new InlineMetaException(FaultCodes.COL_DOCUMENT_MALFORMED, "Expecting header length of 0");
        }

        return new NullMap();
    }

    /**
     * A VERY simple and efficient InlineMetaMap implementation
     * that holds precisely nothing.
     */
    public static class NullMap implements InlineMetaMap {

        private static String[] keys = new String[]{};

        /**
         * @see org.apache.xindice.core.meta.inline.InlineMetaMap#containsKey(String)
         */
        public boolean containsKey(String key) {
            return false;
        }

        /**
         * @see org.apache.xindice.core.meta.inline.InlineMetaMap#get(String)
         */
        public Object get(String key) throws InlineMetaException {
            throw new InlineMetaException(FaultCodes.GEN_CRITICAL_ERROR, "NullMap does not accept key '" + key + "'");
        }

        /**
         * @see org.apache.xindice.core.meta.inline.InlineMetaMap#keys()
         */
        public String[] keys() {
            return keys;
        }

        /**
         * @see org.apache.xindice.core.meta.inline.InlineMetaMap#put(String,Object)
         */
        public void put(String key, Object value) throws InlineMetaException {
            throw new InlineMetaException(FaultCodes.GEN_CRITICAL_ERROR, "NullMap does not accept key '" + key + "'");
        }
    }

}
