/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: AddDocument.java,v 1.10 2003/12/22 14:02:44 vgritsenko Exp $
 */

package org.apache.xindice.tools.command;

import org.apache.xindice.tools.XMLTools;
import org.apache.xindice.client.xmldb.xmlrpc.CollectionImpl;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;
import org.xmldb.api.modules.XMLResource;

import javax.xml.parsers.SAXParserFactory;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Hashtable;

/**
 * AddDocument.java is designed to let the user add Single Documents
 * and/or Multiple Documents to an existing Collection or Nested Collection
 * within the current Database.
 *
 * @version CVS $Revision: 1.10 $, $Date: 2003/12/22 14:02:44 $
 */
public class AddDocument extends Command {

    /**
     * TODO: Description of the Method
     *
     * @param table Description of Parameter
     * @return Description of the Returned Value
     * @exception Exception Description of Exception
     */
    public boolean execute(Hashtable table) throws Exception {

        Collection col = null;
        if (table.get(XMLTools.COLLECTION) == null) {
            System.out.println("ERROR : Collection and switch required");
            return false;
        }

        if (table.get(XMLTools.FILE_PATH) == "") {
            System.out.println("ERROR : File path required");
            return false;
        }

        try {
            // Create a collection instance
            String colstring = normalizeCollectionURI((String) table.get(XMLTools.COLLECTION),
                                                      (String) table.get(XMLTools.LOCAL));
            col = DatabaseManager.getCollection(colstring);
            if (col == null) {
                System.out.println("ERROR : Collection not found!");
                return false;
            }

            // Parse in XML using Xerces
            // TODO: BinaryResource support
            File file = new File((String) table.get(XMLTools.FILE_PATH));
            InputStream fis = new FileInputStream(file);

            SAXParserFactory spf = javax.xml.parsers.SAXParserFactory.newInstance();
            spf.setNamespaceAware(true);

            XMLReader reader = spf.newSAXParser().getXMLReader();
            StringSerializer ser = new StringSerializer(null);
            reader.setContentHandler(ser);
            reader.setProperty("http://xml.org/sax/properties/lexical-handler",
                                  ser);
            reader.parse(new InputSource(fis));
            fis.close();

            // Create the XMLResource and store the document
            XMLResource resource =
                    (XMLResource) col.createResource((String) table.get(XMLTools.NAME_OF),
                                                     "XMLResource");
            resource.setContent(ser.toString());
            col.storeResource(resource);

            System.out.println("Added document " + table.get(XMLTools.COLLECTION) + "/" +
                               resource.getId());
            resource = null;
        } finally {
            if (col != null) {
                col.close();
            }
            col = null;
        }

        return true;
    }

}



