/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: Command.java,v 1.8 2003/08/07 20:13:24 vgritsenko Exp $
 */

package org.apache.xindice.tools.command;

import java.util.Hashtable;

/**
 * Command is the standard interface command line arguments
 *
 * @version CVS $Revision: 1.8 $, $Date: 2003/08/07 20:13:24 $
 */
public abstract class Command {

    // Collection manager path to instantiate
    public static final String XINDICEURI = "xindice://";
    public static final String XINDICELOCALURI = "xindice-embed://";
    public static final String XMLDBURI = "xmldb:";

    // Version of the XML:DB API that we are using
    public static final String XMLDBAPIVERSION = "1.0";

    // Class name of the Standard Xindice Indexer
    public static final String XINDICE_VAL_INDEXER = "org.apache.xindice.core.indexer.ValueIndexer";
    public static final String XINDICE_NAME_INDEXER = "org.apache.xindice.core.indexer.NameIndexer";

    //public boolean execute(Hashtable table, Database db)
    public abstract boolean execute(Hashtable table) throws Exception;

    // public class to return normalized CollectionURI for creation of a Collection object
    public String normalizeCollectionURI(String uri, String local) {
        // Check to see if this uri starts with "xmldb:" , if so treat as absolute
        if (uri.startsWith("xmldb:")) {
            // URI is absolute, leave alone
            return uri;
        } else if (uri.startsWith("xindice:") || uri.startsWith("xindice-embed:")) {
            return (XMLDBURI + uri);
        } else {
            if ((local != null) && local.equals("true")) {
                return (XMLDBURI + XINDICELOCALURI + uri);
            } else {
                // URI passed in is not absoulte, build the full URI
                return (XMLDBURI + XINDICEURI + uri);
            }
        }
    }
}
