/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: RetrieveDocument.java,v 1.9 2003/08/07 20:13:24 vgritsenko Exp $
 */

package org.apache.xindice.tools.command;

import org.apache.xindice.tools.XMLTools;

import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;
import org.xmldb.api.modules.XMLResource;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Hashtable;

/**
 * RetrieveDocument.java is designed to let the user recall or
 * retrieve a Document from a specific Collection and save it to a new location.
 *
 * @version CVS $Revision: 1.9 $, $Date: 2003/08/07 20:13:24 $
 */
public class RetrieveDocument extends Command {

    public boolean execute(Hashtable table) throws Exception {

        Collection col = null;
        try {

            if (table.get(XMLTools.COLLECTION) != null) {
                // Create a collection instance
                String colstring = normalizeCollectionURI((String) table.get(XMLTools.COLLECTION),
                                                          (String) table.get(XMLTools.LOCAL));
                String docname = (String) table.get(XMLTools.NAME_OF);

                col = DatabaseManager.getCollection(colstring);
                // Make sure that the collection was found
                if (col == null) {
                    System.out.println("ERROR : Collection not found!");
                    return false;
                }

                if (table.get(XMLTools.NAME_OF) != null) {

                    XMLResource resource = (XMLResource) col.getResource(docname);
                    // Verify that we were able to find the document
                    if (resource == null) {
                        System.out.println("ERROR : Document not found!");
                        return false;
                    }

                    String documentstr = (String) resource.getContent();

                    if ((documentstr != null) && (table.get(XMLTools.FILE_PATH) != "")) {
                        try {
                            File file = new File((String) table.get(XMLTools.FILE_PATH));
                            // Create the directory structure if necessary
                            if (file.getParentFile() != null) {
                                file.getParentFile().mkdirs();
                            }

                            FileOutputStream out = new FileOutputStream(file);

                            System.out.println("Writing...");

                            out.write(documentstr.getBytes("utf-8"));
                            out.close();

                            System.out.println("Wrote file " + table.get(XMLTools.FILE_PATH));

                        } catch (IOException ie) {
                            System.out.println("ERROR : " + ie);
                            return false;

                        } catch (NullPointerException e) {
                            System.out.println("ERROR : " + e);
                            System.out.println("No file destination given");
                            return false;
                        }
                    } else {
                        // Send to stdout
                        System.out.println(documentstr);
                    }
                } else {
                    System.out.println("ERROR : Document Key required");
                }
            } else {
                System.out.println("ERROR : Collection context required");
            }

        } finally {
            // Release the collection object
            if (col != null) {
                col.close();
            }
        }

        return true;
    }

}

