/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: XPathQueryTest.java,v 1.10 2003/12/06 22:44:36 kevinoneill Exp $
 */

package org.apache.xindice.integration.client.services;

import org.apache.xindice.integration.client.AbstractXmlDbClientTest;
import org.apache.xindice.xml.TextWriter;
import org.custommonkey.xmlunit.XMLAssert;

import org.w3c.dom.Node;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.ResourceIterator;
import org.xmldb.api.base.ResourceSet;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.XMLResource;
import org.xmldb.api.modules.XPathQueryService;

import java.util.ArrayList;
import java.util.List;

/**
 * @version CVS $Revision: 1.10 $, $Date: 2003/12/06 22:44:36 $
 * @author Vladimir R. Bossicard <vladimir@apache.org>
 * @author <a href="mailto:kevinoneill@apache.org">Kevin O'Neill</a>
 */
public class XPathQueryTest extends AbstractXmlDbClientTest
{

	public void setUp() throws Exception
	{
		super.setUp();

		String document1 =
			"<?xml version=\"1.0\"?>"
				+ "<person>"
				+ "<first>John</first>"
				+ "<last>Smith</last>"
				+ "<age>30</age>"
				+ "<phone type=\"work\">555-345-6789</phone>"
				+ "</person>";
		String document2 =
			"<?xml version=\"1.0\"?>"
				+ "<person>"
				+ "<first>Sally</first>"
				+ "<last>Smith</last>"
				+ "<age>20</age>"
				+ "<phone type=\"work\">555-345-6789</phone>"
				+ "</person>";

		this.client.insertDocument(TEST_COLLECTION_PATH, "doc1", document1);
		this.client.insertDocument(TEST_COLLECTION_PATH, "doc2", document2);
	}

	public void tearDown() throws Exception
	{
		this.client.removeDocument(TEST_COLLECTION_PATH, "doc1");
		this.client.removeDocument(TEST_COLLECTION_PATH, "doc2");

		super.tearDown();
	}

	public void testSimpleQuery() throws Exception
	{
		String query = "//person[last='Smith']";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();

		List res = asList(results);
		assertEquals(2, res.size());
	}

	public void testSimpleAndQuery() throws Exception
	{
		String query = "//person[first='John' and last='Smith']";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();

		List res = asList(results);
		assertEquals(1, res.size());
	}

	public void testSimpleOrQuery() throws Exception
	{
		String query = "//person[first='John' or last='Smith']";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();

		List res = asList(results);
		assertEquals(2, res.size());
	}

	public void testMultipleOrQuery() throws Exception
	{
		String query = "//person[first='John' or first='Sally']";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();

		List res = asList(results);
		assertEquals(2, res.size());
	}

	public void testAndOrQuery() throws Exception
	{
		String query = "//person[last='Smith' and (first='John' or first='Sally')]";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();

		List res = asList(results);
		assertEquals(2, res.size());
	}

	public void testSimpleWildcardSearchQuery() throws Exception
	{
		// search all records whose last name contains 'Smi'
		String query = "//person[contains(last, 'Smi')]";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();
		List res = asList(results);
		assertEquals(2, res.size());
	}

	public void testStartsWithSearchQuery() throws Exception
	{
		String document3 =
			"<?xml version=\"1.0\"?>"
				+ "<person>"
				+ "<first>Sally</first>"
				+ "<last>aSm</last>"
				+ "<phone type=\"work\">555-345-6789</phone>"
				+ "</person>";
		this.client.insertDocument(TEST_COLLECTION_PATH, "doc3", document3);

		// search all records whose last name begins with 'Smi'
		String query = "//person[(string-length(//person/last) >= 3) and (substring(//person/last, 1, 3)='Smi')]";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();
		List res = asList(results);
		assertEquals(2, res.size());

		this.client.removeDocument(TEST_COLLECTION_PATH, "doc3");
	}

	public void testEndsWithSearchQuery() throws Exception
	{
		// search all records whose last name begins with 'ith'
		String query = "//person[(string-length(//person/last) >= 4) and (substring(//person/last, 2)='mith')]";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();
		List res = asList(results);
		assertEquals(2, res.size());
	}

	public void testGreaterSearchQuery() throws Exception
	{
		String query = "//person[age > 25]";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();
		List res = asList(results);
		assertEquals(1, res.size());
	}

	public void testGetSingleTextNode() throws Exception
	{
		String query = "//person[first='John' and last='Smith']/first/text()";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		ResourceSet resultSet = xpathservice.query(query);

		assertEquals(1L, resultSet.getSize());
		assertEquals("John", resultSet.getResource(0).getContent());
	}

	public void testGetMultipleTextNodes() throws Exception
	{
		String query = "//person/first/text()";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();

		List res = asList(results);
		assertEquals(2, res.size());
	}

	public void testNamespaceDOM() throws Exception
	{
		String document3 =
			"<?xml version=\"1.0\"?>"
				+ "<p:person xmlns:p='http://example.net/person' >"
				+ "<p:first>Sally</p:first>"
				+ "<p:last>Smith</p:last>"
				+ "<p:phone type=\"work\">555-345-6789</p:phone>"
				+ "</p:person>";
		this.client.insertDocument(TEST_COLLECTION_PATH, "doc3", document3);

		String query = "//h:person[h:first='Sally' and h:last='Smith']/h:first";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		xpathservice.setNamespace("h", "http://example.net/person");

		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();
		List res = asList(results);
		assertEquals(1, res.size());

		XMLResource resource = (XMLResource) resultSet.getResource(0);

		// ensure that the resource has the correct doc id.
		assertEquals("doc3", resource.getDocumentId());

		Node node = resource.getContentAsDOM();

		// add source node information to the compared xml as it's added by
		// the query processor.
		XMLAssert.assertXMLEqual(
			"<first xmlns:src='http://xml.apache.org/xindice/Query' src:col='/db/testing/current' src:key='doc3' xmlns='http://example.net/person'>Sally</first>",
			TextWriter.toString(node));

		this.client.removeDocument(TEST_COLLECTION_PATH, "doc3");
	}

	public void testFetchMultipleAttributeNode() throws Exception
	{
		String document3 = "<?xml version=\"1.0\"?>" + "<foo bar='foo'>" + "<bar foo='bar' bar='bar'/>" + "</foo>";

		client.insertDocument(TEST_COLLECTION_PATH, "doc3", document3);

		String query = "//@bar";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");

		ResourceSet resultSet = null;
		try
		{
			resultSet = xpathservice.query(query);
		}
		catch (Exception e)
		{
			fail(e.getMessage());
		}

		ResourceIterator results = resultSet.getIterator();
		List res = asList(results);
		assertEquals(2, res.size());

		XMLResource resource = (XMLResource) resultSet.getResource(0);

		Node node = resource.getContentAsDOM();

		client.removeDocument(TEST_COLLECTION_PATH, "doc3");
	}

	public void testFetchAttributeNode() throws Exception
	{
		String document3 = "<?xml version=\"1.0\"?>" + "<foo bar='foo'>" + "<bar foo='bar' bar='bar'/>" + "</foo>";

		client.insertDocument(TEST_COLLECTION_PATH, "doc3", document3);

		String query = "//foo/@bar";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");

		ResourceSet resultSet = null;
		try
		{
			resultSet = xpathservice.query(query);
		}
		catch (Exception e)
		{
			fail(e.getMessage());
		}

		assertEquals(1L, resultSet.getSize());
		assertEquals("foo", resultSet.getResource(0).getContent().toString());
	}

	public void testFetchBoolean() throws Exception
	{
		String query = "boolean(//person)";

		Collection col = client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		try
		{
			ResourceSet resultSet = xpathservice.query(query);

			// when we do support it the following tests will be run
			assertEquals(2L, resultSet.getSize());
			assertEquals("true", resultSet.getResource(1L).getContent());

		}
		catch (XMLDBException e)
		{
			fail(e.getMessage());
		}
	}

	public void testFetchString() throws Exception
	{
		String query = "string(//person[first='John' and last='Smith']/first)";

		Collection col = client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		try
		{
			ResourceSet resultSet = xpathservice.query(query);

			assertEquals(2L, resultSet.getSize());
		}
		catch (XMLDBException e)
		{
			fail(e.getMessage());
		}
	}

	public void testFetchNumber() throws Exception
	{
		String query = "count(//person[first='John' and last='Smith'])";

		Collection col = client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		try
		{
			ResourceSet resultSet = xpathservice.query(query);

			assertEquals(2L, resultSet.getSize());
		}
		catch (XMLDBException e)
		{
			fail(e.getMessage());
		}
	}

	public void testSrcAddedToRootOnly() throws Exception
	{
		String query = "//person[first='John' and last='Smith']";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();

		List res = asList(results);
		assertEquals(1, res.size());

		XMLResource resource = (XMLResource) resultSet.getResource(0);

		String john =
			"<?xml version=\"1.0\"?>"
				+ "<person xmlns:src='http://xml.apache.org/xindice/Query' src:col='/db/testing/current' src:key='doc1'>"
				+ "<first>John</first>"
				+ "<last>Smith</last>"
				+ "<age>30</age>"
				+ "<phone type=\"work\">555-345-6789</phone>"
				+ "</person>";

		// ensure that the resource has the correct doc id.
		assertEquals("doc1", resource.getDocumentId());

		XMLAssert.assertXMLEqual(john, TextWriter.toString(resource.getContentAsDOM()));
	}

	public void testConflictingPrefix() throws Exception
	{
		String document3 =
			"<?xml version=\"1.0\"?>"
				+ "<src:person xmlns:src='http://example.net/person' >"
				+ "<src:first>Sally</src:first>"
				+ "<src:last>Smith</src:last>"
				+ "<src:phone type=\"work\">555-345-6789</src:phone>"
				+ "</src:person>";
		this.client.insertDocument(TEST_COLLECTION_PATH, "doc3", document3);

		String query = "//h:person[h:first='Sally' and h:last='Smith']/h:first";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		xpathservice.setNamespace("h", "http://example.net/person");

		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();
		List res = asList(results);
		assertEquals(1, res.size());

		XMLResource resource = (XMLResource) resultSet.getResource(0);

		// ensure that the resource has the correct doc id.
		assertEquals("doc3", resource.getDocumentId());

		Node node = resource.getContentAsDOM();

		// add source node information to the compared xml as it's added by
		// the query processor.
		XMLAssert.assertXMLEqual(
			"<first xmlns:src='http://xml.apache.org/xindice/Query' src:col='/db/testing/current' src:key='doc3' xmlns='http://example.net/person'>Sally</first>",
			TextWriter.toString(node));

		this.client.removeDocument(TEST_COLLECTION_PATH, "doc3");
	}

	public static List asList(ResourceIterator iter) throws Exception
	{
		if (iter == null)
		{
			return null;
		}

		List result = null;
		result = new ArrayList();
		while (iter.hasMoreResources())
		{
			result.add(iter.nextResource());
		}
		return result;
	}

	public void testNamespacesNotImportedDeep() throws Exception
	{
		// make sure that we don't have the namespace defs needlessly
		// imported from the root all the way through the tree
		// i.e. namespaces imported from above root should not
		// be spread into child nodes
		String document3 =
			"<?xml version=\"1.0\"?>"
				+ "<p:person xmlns:p='http://example.net/person' >"
				+ "<p:first>Sally</p:first>"
				+ "<p:last>Smith</p:last>"
				+ "<p:phone p:type=\"work\">555-345-6789</p:phone>"
				+ "</p:person>";
		this.client.insertDocument(TEST_COLLECTION_PATH, "doc3", document3);

		String query = "//h:person[h:first='Sally' and h:last='Smith']";

		Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
		XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
		xpathservice.setNamespace("h", "http://example.net/person");

		ResourceSet resultSet = xpathservice.query(query);

		ResourceIterator results = resultSet.getIterator();
		List res = asList(results);
		assertEquals(1, res.size());

		XMLResource resource = (XMLResource) resultSet.getResource(0);
		Node node = resource.getContentAsDOM();
		String retrieved = TextWriter.toString(node);
		assertFalse("Namespace definitions imported deep in: " + retrieved, -1 == retrieved.indexOf("<p:first>"));

		// ensure that the resource has the correct doc id.
		assertEquals("doc3", resource.getDocumentId());

		// add source node information to the compared xml as it's added by
		// the query processor. Note, this should work using without the
		// prefixes in the control document using the default namespace,
		// (i.e. xmlns='http://example.net/person') but I couldn't get it to work
		// with XMLAssert (and we'return not testing XMLAssert so we just use the prefixes here too)
		XMLAssert.assertXMLEqual(
			"<p:person xmlns:p='http://example.net/person' xmlns:src='http://xml.apache.org/xindice/Query' src:col='/db/testing/current' src:key='doc3'>"
				+ "<p:first>Sally</p:first>"
				+ "<p:last>Smith</p:last>"
				+ "<p:phone p:type=\"work\">555-345-6789</p:phone>"
				+ "</p:person>",
			TextWriter.toString(node));

		this.client.removeDocument(TEST_COLLECTION_PATH, "doc3");
	}

	public void testMathOpQuery() throws Exception
	{
		// force a non-XObject Object to be supplied to evalMathOperation as
		// one of its operands
		// this works in the buggy version of XPathQueryResolver but only
		// because a ClassCastException that cuts short the indexing evaluation
		// is caught and ignored forcing a collection scan (if logging WARN, you'll see stack trace)

		String query = "//person[substring(last, string-length(last) - two)='ith']";
		String document3 =
			"<?xml version=\"1.0\"?>"
				+ "<person>"
				+ "<first>Sally</first>"
				+ "<last>Smith</last>"
				+ "<one>1</one>"
				+ "<two>2.0</two>"
				+ "<phone call=\"no\" type=\"work\">555-345-6789</phone>"
				+ "</person>";
		this.client.insertDocument(TEST_COLLECTION_PATH, "doc3", document3);

		try
		{
			Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
			XPathQueryService xpathservice = (XPathQueryService) col.getService("XPathQueryService", "1.0");
			ResourceSet resultSet = xpathservice.query(query);

			ResourceIterator results = resultSet.getIterator();

			List res = asList(results);
			assertEquals(1, res.size());
		}
		finally
		{
			this.client.removeDocument(TEST_COLLECTION_PATH, "doc3");
		}
	}
}
