/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: XUpdateQueryTest.java,v 1.5 2003/08/07 20:13:27 vgritsenko Exp $
 */

package org.apache.xindice.integration.client.services;

import org.apache.xindice.integration.client.AbstractXmlDbClientTest;

import org.xmldb.api.modules.XUpdateQueryService;
import org.xmldb.api.base.Collection;

/**
 * @version CVS $Revision: 1.5 $, $Date: 2003/08/07 20:13:27 $
 * @author Vladimir R. Bossicard <vladimir@apache.org>
 */
public class XUpdateQueryTest
        extends AbstractXmlDbClientTest {

    public void setUp()
            throws Exception {
        super.setUp();

        String document1 = "<?xml version=\"1.0\"?>" +
                "<person status=\"single\">" +
                "<first>John</first>" +
                "<last>Smith</last>" +
                "<phone type=\"work\">555-345-6789</phone>" +
                "</person>";
        String document2 = "<?xml version=\"1.0\"?>" +
                "<person status=\"married\">" +
                "<first>Sally</first>" +
                "<last>Benton</last>" +
                "<phone type=\"work\">555-345-6789</phone>" +
                "</person>";

        this.client.insertDocument(TEST_COLLECTION_PATH, "doc1", document1);
        this.client.insertDocument(TEST_COLLECTION_PATH, "doc2", document2);
    }

    public void tearDown()
            throws Exception {
        this.client.removeDocument(TEST_COLLECTION_PATH, "doc1");
        this.client.removeDocument(TEST_COLLECTION_PATH, "doc2");

        super.tearDown();
    }

    public void testUpdateDocument()
            throws Exception {
        String query =
                "<xupdate:modifications version=\"1.0\" xmlns:xupdate=\"http://www.xmldb.org/xupdate\">" +
                "   <xupdate:update select=\"/person/@status\">married</xupdate:update>" +
                "   <xupdate:update select=\"/person/first\">Ben</xupdate:update>" +
                "   <xupdate:update select=\"/person/phone[@type = 'work']\">480-300-3003</xupdate:update>" +
                "</xupdate:modifications>";

        String updatedDocument = "<?xml version=\"1.0\"?>\n" +
                "<person status=\"married\">" +
                "<first>Ben</first>" +
                "<last>Smith</last>" +
                "<phone type=\"work\">480-300-3003</phone>" +
                "</person>";

        Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
        XUpdateQueryService service = (XUpdateQueryService) col.getService("XUpdateQueryService", "1.0");

        long count = service.updateResource("doc1", query);
        assertEquals(1, count);

        String doc = this.client.getDocument(TEST_COLLECTION_PATH, "doc1");
        assertNotNull(doc);
        assertEquals(updatedDocument, doc);
    }

    public void testUpdateCollection()
            throws Exception {
        String query =
                "<xupdate:modifications version=\"1.0\" xmlns:xupdate=\"http://www.xmldb.org/xupdate\">" +
                "   <xupdate:update select=\"/person/@status\">divorced</xupdate:update>" +
                "   <xupdate:update select=\"/person/first\">Ben</xupdate:update>" +
                "   <xupdate:update select=\"/person/phone[@type = 'work']\">480-300-3003</xupdate:update>" +
                "</xupdate:modifications>";

        String updatedDocument1 = "<?xml version=\"1.0\"?>\n" +
                "<person status=\"divorced\">" +
                "<first>Ben</first>" +
                "<last>Smith</last>" +
                "<phone type=\"work\">480-300-3003</phone>" +
                "</person>";
        String updatedDocument2 = "<?xml version=\"1.0\"?>\n" +
                "<person status=\"divorced\">" +
                "<first>Ben</first>" +
                "<last>Benton</last>" +
                "<phone type=\"work\">480-300-3003</phone>" +
                "</person>";

        Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
        XUpdateQueryService service = (XUpdateQueryService) col.getService("XUpdateQueryService", "1.0");

        long count = service.update(query);
        assertEquals(6, count);

        String doc = this.client.getDocument(TEST_COLLECTION_PATH, "doc1");
        assertNotNull(doc);
        assertEquals(updatedDocument1, doc);

        doc = this.client.getDocument(TEST_COLLECTION_PATH, "doc2");
        assertNotNull(doc);
        assertEquals(updatedDocument2, doc);
    }

}
